import React, { useEffect, useState } from "react";
import CopyToClipboard from "react-copy-to-clipboard";
import { Prism as SyntaxHighlighter } from 'react-syntax-highlighter';
import style from "react-syntax-highlighter/dist/cjs/styles/prism/vsc-dark-plus"
import cutomStyle from "./customStyle";
import getLangFromFilename from "./getLangFromFilename";

interface Props {
  shareID: string;
}

const filenameRegex = /filename\*?=['"]?(?:UTF-\d['"]*)?([^;\r\n"']*)['"]?;?/gi;

function parseFilename(headers: Headers): string{
  const raw = headers.get("content-disposition");
  if (raw == null) {
    return ""
  }

  let m;
  while ((m = filenameRegex.exec(raw)) !== null) {
    // Return the first match that we got
    return m[1];
  }
  return "";
}

function App(props: Props) {
  const [content,setContent] = useState<string>("");
  const [title,setTitle] = useState<string>("");
  const [lang,setLang] = useState<string>("text");

  useEffect(()=>{
    (async ()=>{
      const response = await fetch(`/index.php/s/${props.shareID}/download`);

      if (response.status === 404){
        // File not found
        // TODO display error
        return;
      }

      const contentLength = parseInt(response.headers.get("content-length")!,10);
      const filename = parseFilename(response.headers);
      setTitle(filename);
      document.title = filename;

      setLang(getLangFromFilename(filename) || "text")

      if (contentLength <= 20000) {
        const text = await response.text()
        setContent(text);

      }else{
        // TODO display error
      }
    })()
  },[props.shareID])

  function downloadClick() {
    window.open(`/index.php/s/${props.shareID}/download`, "_self");
  }

  return (
      <div className="fileContainer">
        <div className="headerContainer">
          <span className="fileTitle">{title}</span>
          <div className="buttonContainer">
            <CopyToClipboard text={content}>
              <button className="titleButton">Copy</button>
            </CopyToClipboard>
            <button className="titleButton" onClick={downloadClick}>Download</button>
          </div>
        </div>
        
        <SyntaxHighlighter
          customStyle={cutomStyle} 
          showLineNumbers={true}
          style={style}
          wrapLongLines={true}
          language={lang}
        >
          {content}
      </SyntaxHighlighter>
      </div>
  );
}

export default App;
